@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -476,9 +475,9 @@ string_alloc_buf (const char *str, struct gc_arena *gc)
   ASSERT (str);
 
 #ifdef DMALLOC
-  buf_set_read (&buf, string_alloc_debug (str, NULL, file, line), strlen (str) + 1);
+  buf_set_read (&buf, string_alloc_debug (str, gc, file, line), strlen (str) + 1);
 #else
-  buf_set_read (&buf, string_alloc (str, NULL), strlen (str) + 1);
+  buf_set_read (&buf, string_alloc (str, gc), strlen (str) + 1);
 #endif
 
   if (buf.len > 0) /* Don't count trailing '\0' as part of length */
@@ -513,11 +512,11 @@ buf_string_compare_advance (struct buffer *src, const char *match)
 }
 
 int
-buf_substring_len (const struct buffer *buf, char delim)
+buf_substring_len (const struct buffer *buf, int delim)
 {
   int i = 0;
   struct buffer tmp = *buf;
-  char c;
+  int c;
 
   while ((c = buf_read_u8 (&tmp)) >= 0)
     {
