@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1288,6 +1287,16 @@ multi_connection_established (struct multi_context *m, struct multi_instance *mi
 	    cc_succeeded = false;
 	}
 
+      /*
+       * Check for "disable" directive in client-config-dir file
+       * or config file generated by --client-connect script.
+       */
+      if (mi->context.options.disable)
+	{
+	  msg (D_MULTI_ERRORS, "MULTI: client has been rejected due to 'disable' directive");
+	  cc_succeeded = false;
+	}
+
       if (cc_succeeded)
 	{
 	  /*
@@ -2024,6 +2033,14 @@ management_callback_kill_by_addr (void *arg, const in_addr_t addr, const int por
   return count;
 }
 
+static void
+management_delete_event (void *arg, event_t event)
+{
+  struct multi_context *m = (struct multi_context *) arg;
+  if (m->mtcp)
+    multi_tcp_delete_event (m->mtcp, event);
+}
+
 #endif
 
 void
@@ -2039,6 +2056,7 @@ init_management_callback_multi (struct multi_context *m)
       cb.show_net = management_show_net_callback;
       cb.kill_by_cn = management_callback_kill_by_cn;
       cb.kill_by_addr = management_callback_kill_by_addr;
+      cb.delete_event = management_delete_event;
       management_set_callback (management, &cb);
     }
 #endif
@@ -2070,126 +2088,6 @@ tunnel_server (struct context *top)
   }
 }
 
-#ifdef USE_PTHREAD
-
-/*
- * Multithreading support
- */
-void
-inherit_multi_context (struct multi_context *dest, const struct multi_context *src, const unsigned int thread_mode)
-{
-  ASSERT (thread_mode & MC_WORK_THREAD);
-
-  *dest = *src;
-  dest->thread_mode = thread_mode;
-  dest->pending = NULL;
-  dest->earliest_wakeup = NULL;
-  dest->mpp_touched = NULL;
-  dest->per_second_trigger = 0;
-  multi_top_init (dest, &src->top, true);
-}
-
-void
-multi_acquire_io_lock (struct multi_context *m, const unsigned int iow_flags)
-{
-  dmsg (D_THREAD_DEBUG, "THREAD: acquire I/O lock ENTRY, flags=0x%08x", iow_flags);
-  if ((iow_flags & IOW_READ) && !m->thread_local.read_owned)
-    {
-      mutex_lock (&m->thread_shared->read.mutex);
-      m->thread_local.read_owned = true;
-    }
-  else
-    {
-      if ((iow_flags & (IOW_TO_LINK | IOW_MBUF)) && !m->thread_local.write_link_owned)
-	{
-	  mutex_lock (&m->thread_shared->write_link.mutex);
-	  m->thread_local.write_link_owned = true;
-	}
-      if ((iow_flags & IOW_TO_TUN) && !m->thread_local.write_tun_owned)
-	{
-	  mutex_lock (&m->thread_shared->write_tun.mutex);
-	  m->thread_local.write_tun_owned = true;
-	}
-    }
-  dmsg (D_THREAD_DEBUG, "THREAD: acquire I/O lock EXIT, flags=0x%08x", iow_flags);
-}
-
-void
-multi_release_io_lock (struct multi_context *m)
-{
-  dmsg (D_THREAD_DEBUG, "THREAD: release I/O lock");
-  if (m->thread_local.read_owned)
-    {
-      mutex_unlock (&m->thread_shared->read.mutex);
-      m->thread_local.read_owned = false;
-    }
-  if (m->thread_local.write_link_owned)
-    {
-      mutex_unlock (&m->thread_shared->write_link.mutex);
-      m->thread_local.write_link_owned = false;
-    }
-  if (m->thread_local.write_tun_owned)
-    {
-      mutex_unlock (&m->thread_shared->write_tun.mutex);
-      m->thread_local.write_tun_owned = false;
-    }
-}
-
-void
-thread_shared_init (struct multi_context_thread_shared *ts)
-{
-  CLEAR (*ts);
-  mutex_init (&ts->read.mutex);
-  mutex_init (&ts->write_link.mutex);
-  mutex_init (&ts->write_tun.mutex);
-}
-
-void
-thread_shared_uninit (struct multi_context_thread_shared *ts)
-{
-  mutex_destroy (&ts->read.mutex);
-  mutex_destroy (&ts->write_link.mutex);
-  mutex_destroy (&ts->write_tun.mutex);
-}
-
-void
-multi_set_pending (struct multi_context *m, struct multi_instance *mi)
-{
-  if (mi)
-    {
-      if (mi != m->thread_local.held)
-	{
-	  if (m->thread_local.held)
-	    {
-	      mutex_unlock (&m->thread_local.held->mutex);
-	      m->thread_local.held = NULL;
-	    }
-	  if (mutex_trylock (&mi->mutex))
-	    {
-	      m->thread_local.held = mi;
-	      m->pending = mi;
-	      dmsg (D_THREAD_DEBUG, "THREAD: instance lock SUCCEEDED");
-	    }
-	  else
-	    {
-	      m->pending = NULL;
-	      dmsg (D_THREAD_DEBUG, "THREAD: instance lock FAILED");
-	    }
-	}
-    }
-  else
-    {
-      if (m->thread_local.held)
-	{
-	  mutex_unlock (&m->thread_local.held->mutex);
-	  m->thread_local.held = NULL;
-	}
-      m->pending = NULL;
-    }
-}
-
-#endif /* USE_PTHREAD */
-
 #else
 static void dummy(void) {}
 #endif /* P2MP_SERVER */
