@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,7 +39,7 @@
 
 #include "openvpn-plugin.h"
 
-#define DEBUG(verb) ((verb) >= 7) // JYFIXME
+#define DEBUG(verb) ((verb) >= 7)
 
 /* Command codes for foreground -> background communication */
 #define COMMAND_RUN_SCRIPT 0
@@ -487,7 +486,7 @@ down_root_server (const int fd, char *command, const char *argv[], const char *e
 
 	case -1:
 	  fprintf (stderr, "DOWN-ROOT: BACKGROUND: read error on command channel\n");
-	  goto done;
+	  break;
 
 	default:
 	  fprintf (stderr, "DOWN-ROOT: BACKGROUND: unknown command code: code=%d, exiting\n",
