@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2003 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,71 +22,185 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifndef OPENVPN_SSL_H
+#define OPENVPN_SSL_H
+
 #if defined(USE_CRYPTO) && defined(USE_SSL)
 
 #include <openssl/ssl.h>
 #include <openssl/bio.h>
 #include <openssl/rand.h>
 #include <openssl/err.h>
+#include <openssl/pkcs12.h>
+#include <openssl/x509v3.h>
 
 #include "basic.h"
+#include "common.h"
 #include "crypto.h"
 #include "packet_id.h"
 #include "session_id.h"
 #include "reliable.h"
 #include "socket.h"
 #include "mtu.h"
+#include "thread.h"
+#include "options.h"
+#include "plugin.h"
 
 /*
- * Openvpn Protocol.
+ * OpenVPN Protocol, taken from ssl.h in OpenVPN source code.
  *
- * TCP/UDP Packet:
- *   packet length (16 bits, unsigned) -- TCP only, always sent as plaintext
- *   packet opcode (high 5 bits, see P_ constants below)
- *   key_id (low 3 bits, see key_id in struct tls_session below for comment)
- *   payload (n bytes)
+ * TCP/UDP Packet:  This represents the top-level encapsulation.
  *
- * P_CONTROL* and P_ACK Payload:
- *   session_id (random 64 bit value to identify session)
- *   hmac for authentication (usually 16 or 20 bytes)
+ * TCP/UDP packet format:
+ *
+ *   Packet length (16 bits, unsigned) -- TCP only, always sent as
+ *       plaintext.  Since TCP is a stream protocol, the packet
+ *       length words define the packetization of the stream.
+ *
+ *   Packet opcode/key_id (8 bits) -- TLS only, not used in
+ *       pre-shared secret mode.
+ *            packet message type, a P_* constant (high 5 bits)
+ *            key_id (low 3 bits, see key_id in struct tls_session
+ *              below for comment).  The key_id refers to an
+ *              already negotiated TLS session.  OpenVPN seamlessly
+ *              renegotiates the TLS session by using a new key_id
+ *              for the new session.  Overlap (controlled by
+ *              user definable parameters) between old and new TLS
+ *              sessions is allowed, providing a seamless transition
+ *              during tunnel operation.
+ *
+ *   Payload (n bytes), which may be a P_CONTROL, P_ACK, or P_DATA
+ *       message.
+ *
+ * Message types:
+ *
+ *  P_CONTROL_HARD_RESET_CLIENT_V1 -- Key method 1, initial key from
+ *    client, forget previous state.
+ *
+ *  P_CONTROL_HARD_RESET_SERVER_V1 -- Key method 2, initial key
+ *    from server, forget previous state.
+ *
+ *  P_CONTROL_SOFT_RESET_V1 -- New key, with a graceful transition
+ *    from old to new key in the sense that a transition window
+ *    exists where both the old or new key_id can be used.  OpenVPN
+ *    uses two different forms of key_id.  The first form is 64 bits
+ *    and is used for all P_CONTROL messages.  P_DATA messages on the
+ *    other hand use a shortened key_id of 3 bits for efficiency
+ *    reasons since the vast majority of OpenVPN packets in an
+ *    active tunnel will be P_DATA messages.  The 64 bit form
+ *    is referred to as a session_id, while the 3 bit form is
+ *    referred to as a key_id.
+ *
+ *  P_CONTROL_V1 -- Control channel packet (usually TLS ciphertext).
+ *
+ *  P_ACK_V1 -- Acknowledgement for P_CONTROL packets received.
+ *
+ *  P_DATA_V1 -- Data channel packet containing actual tunnel data
+ *    ciphertext.
+ *
+ *  P_CONTROL_HARD_RESET_CLIENT_V2 -- Key method 2, initial key from
+ *   client, forget previous state.
+ *
+ *  P_CONTROL_HARD_RESET_SERVER_V2 -- Key method 2, initial key from
+ *   server, forget previous state.
+ *
+ * P_CONTROL* and P_ACK Payload:  The P_CONTROL message type
+ * indicates a TLS ciphertext packet which has been encapsulated
+ * inside of a reliability layer.  The reliability layer is
+ * implemented as a straightforward ACK and retransmit model.
+ *
+ * P_CONTROL message format:
+ *
+ *   local session_id (random 64 bit value to identify TLS session).
+ *   HMAC signature of entire encapsulation header for integrity
+ *       check if --tls-auth is specified (usually 16 or 20 bytes).
  *   packet-id for replay protection (4 or 8 bytes, includes
- *     sequence number and optional time_t timestamp)
- *   acknowledge packet_id array length (1 byte)
- *   acknowledge packet-id array (if length > 0)
- *   acknowledge remote session_id (if length > 0)
- *   control packet-id (4 bytes)
- *   TLS ciphertext (n bytes) (only for P_CONTROL)
+ *       sequence number and optional time_t timestamp).
+ *   P_ACK packet_id array length (1 byte).
+ *   P_ACK packet-id array (if length > 0).
+ *   P_ACK remote session_id (if length > 0).
+ *   message packet-id (4 bytes).
+ *   TLS payload ciphertext (n bytes) (only for P_CONTROL).
  *
- * TLS plaintext packet (key_method == 1):
- *   cipher key length in bytes (1 byte)
- *   cipher key (n bytes)
- *   hmac key length in bytes (1 byte)
- *   hmac key (n bytes)
- *   options string (n bytes, null terminated, client/server options string must match)
+ * Once the TLS session has been initialized and authenticated,
+ * the TLS channel is used to exchange random key material for
+ * bidirectional cipher and HMAC keys which will be
+ * used to secure actual tunnel packets.  OpenVPN currently
+ * implements two key methods.  Key method 1 directly
+ * derives keys using random bits obtained from the RAND_bytes
+ * OpenSSL function.  Key method 2 mixes random key material
+ * from both sides of the connection using the TLS PRF mixing
+ * function.  Key method 2 is the preferred method and is the default
+ * for OpenVPN 2.0.
+ * 
+ * TLS plaintext content:
  *
- * TLS plaintext packet (key_method >= 2):
- *   0 (4 bytes)
- *   key_method (1 byte)
- *   key_source structure (pre_master only defined for client -> server)
- *   options_string_length, including null (2 bytes)
- *   options string (n bytes, null terminated, client/server options string must match)
+ * TLS plaintext packet (if key_method == 1):
  *
- * P_DATA Payload:
- *   hmac of ciphertext IV + ciphertext (if enabled by --auth)
- *   ciphertext IV (size is cipher-dependent, if not disabled by --no-iv)
- *   P_DATA ciphertext
+ *   Cipher key length in bytes (1 byte).
+ *   Cipher key (n bytes).
+ *   HMAC key length in bytes (1 byte).
+ *   HMAC key (n bytes).
+ *   Options string (n bytes, null terminated, client/server options
+ *       string should match).
+ *
+ * TLS plaintext packet (if key_method == 2):
+ *
+ *   Literal 0 (4 bytes).
+ *   key_method type (1 byte).
+ *   key_source structure (pre_master only defined for client ->
+ *       server).
+ *   options_string_length, including null (2 bytes).
+ *   Options string (n bytes, null terminated, client/server options
+ *       string must match).
+ *   [The username/password data below is optional, record can end
+ *       at this point.]
+ *   username_string_length, including null (2 bytes).
+ *   Username string (n bytes, null terminated).
+ *   password_string_length, including null (2 bytes).
+ *   Password string (n bytes, null terminated).
+ *
+ * The P_DATA payload represents encrypted, encapsulated tunnel
+ * packets which tend to be either IP packets or Ethernet frames.
+ * This is essentially the "payload" of the VPN.
+ *
+ * P_DATA message content:
+ *   HMAC of ciphertext IV + ciphertext (if not disabled by
+ *       --auth none).
+ *   Ciphertext IV (size is cipher-dependent, if not disabled by
+ *       --no-iv).
+ *   Tunnel packet ciphertext.
  *
  * P_DATA plaintext
- *   packet_id (4 or 8 bytes, if not disabled by --no-replay)
- *   user plaintext (n bytes)
+ *   packet_id (4 or 8 bytes, if not disabled by --no-replay).
+ *       In SSL/TLS mode, 4 bytes are used because the implementation
+ *       can force a TLS renegotation before 2^32 packets are sent.
+ *       In pre-shared key mode, 8 bytes are used (sequence number
+ *       and time_t value) to allow long-term key usage without
+ *       packet_id collisions.
+ *   User plaintext (n bytes).
  *
  * Notes:
- *   (1) Acknowledgements can be encoded in either the dedicated P_ACK record
- *       or they can be prepended to a P_CONTROL* record.
- *   (2) P_DATA and P_CONTROL/P_ACK use independent packet-id sequences because
- *       P_DATA is an unreliable channel while P_CONTROL/P_ACK is a reliable channel.
+ *   (1) ACK messages can be encoded in either the dedicated
+ *       P_ACK record or they can be prepended to a P_CONTROL message.
+ *   (2) P_DATA and P_CONTROL/P_ACK use independent packet-id
+ *       sequences because P_DATA is an unreliable channel while
+ *       P_CONTROL/P_ACK is a reliable channel.  Each use their
+ *       own independent HMAC keys.
+ *   (3) Note that when --tls-auth is used, all message types are
+ *       protected with an HMAC signature, even the initial packets
+ *       of the TLS handshake.  This makes it easy for OpenVPN to
+ *       throw away bogus packets quickly, without wasting resources
+ *       on attempting a TLS handshake which will ultimately fail.
  */
 
+/* Used in the TLS PRF function */
+#define KEY_EXPANSION_ID "OpenVPN"
+
+/* passwords */
+#define UP_TYPE_AUTH        "Auth"
+#define UP_TYPE_PRIVATE_KEY "Private Key"
+
 /* packet opcode (high 5 bits) and key-id (low 3 bits) are combined in one byte */
 #define P_KEY_ID_MASK                  0x07
 #define P_OPCODE_SHIFT                 3
@@ -162,7 +275,17 @@
  * Buffer sizes (also see mtu.h).
  */
 
-#define PLAINTEXT_BUFFER_SIZE 1024
+#define PLAINTEXT_BUFFER_SIZE TLS_CHANNEL_BUF_SIZE
+
+/* Maximum length of common name */
+#define TLS_CN_LEN 64
+
+/* Legal characters in an X509 or common name */
+#define X509_NAME_CHAR_CLASS   (CC_ALNUM|CC_UNDERBAR|CC_DASH|CC_DOT|CC_AT|CC_COLON|CC_SLASH|CC_EQUAL)
+#define COMMON_NAME_CHAR_CLASS (CC_ALNUM|CC_UNDERBAR|CC_DASH|CC_DOT|CC_AT)
+
+/* Maximum length of OCC options string passed as part of auth handshake */
+#define TLS_OPTIONS_LEN 512
 
 /*
  * Range of key exchange methods
@@ -170,6 +293,9 @@
 #define KEY_METHOD_MIN 1
 #define KEY_METHOD_MAX 2
 
+/* key method taken from lower 4 bits */
+#define KEY_METHOD_MASK 0x0F
+
 /*
  * Measure success rate of TLS handshakes, for debugging only
  */
@@ -224,17 +350,23 @@ struct key_state
 
   struct key_ctx_bi key;	       /* data channel keys for encrypt/decrypt/hmac */
 
-  struct key_source2 key_src;          /* source entropy for key expansion */
+  struct key_source2 *key_src;         /* source entropy for key expansion */
 
   struct buffer plaintext_read_buf;
   struct buffer plaintext_write_buf;
   struct buffer ack_write_buf;
-  struct reliable send_reliable; /* holds a copy of outgoing packets until ACK received */
-  struct reliable rec_reliable;	 /* order incoming ciphertext packets before we pass to TLS */
-  struct reliable_ack rec_ack;	 /* buffers all packet IDs we want to ACK back to sender */
+
+  struct reliable *send_reliable; /* holds a copy of outgoing packets until ACK received */
+  struct reliable *rec_reliable;  /* order incoming ciphertext packets before we pass to TLS */
+  struct reliable_ack *rec_ack;	  /* buffers all packet IDs we want to ACK back to sender */
 
   int n_bytes;			 /* how many bytes sent/recvd since last key exchange */
   int n_packets;		 /* how many packets sent/recvd since last key exchange */
+
+  /*
+   * If bad username/password, TLS connection will come up but 'authenticated' will be false.
+   */
+  bool authenticated;
 };
 
 /*
@@ -252,16 +384,20 @@ struct tls_options
   /* true if we are a TLS server, client otherwise */
   bool server;
 
+#ifdef ENABLE_OCC
   /* local and remote options strings
      that must match between client and server */
   const char *local_options;
   const char *remote_options;
+#endif
 
   /* from command line */
   int key_method;
   bool replay;
   bool single_session;
+#ifdef ENABLE_OCC
   bool disable_occ;
+#endif
   int transition_window;
   int handshake_window;
   interval_t packet_timeout;
@@ -269,8 +405,19 @@ struct tls_options
   int renegotiate_packets;
   interval_t renegotiate_seconds;
 
-  /* use 32 bit or 64 bit packet-id? */
-  bool packet_id_long_form;
+  /* cert verification parms */
+  const char *verify_command;
+  const char *verify_x509name;
+  const char *crl_file;
+  int ns_cert_type;
+
+  /* allow openvpn config info to be
+     passed over control channel */
+  bool pass_config_info;
+
+  /* struct crypto_option flags */
+  unsigned int crypto_flags_and;
+  unsigned int crypto_flags_or;
 
   int replay_window;                   /* --replay-window parm */
   int replay_time;                     /* --replay-window parm */
@@ -281,6 +428,22 @@ struct tls_options
 
   /* frame parameters for TLS control channel */
   struct frame frame;
+
+  /* used for username/password authentication */
+  const char *auth_user_pass_verify_script;
+  bool auth_user_pass_verify_script_via_file;
+  const char *tmp_dir;
+  bool username_as_common_name;
+
+  /* use the client-config-dir as a positive authenticator */
+  const char *client_config_dir_exclusive;
+
+  /* instance-wide environment variable set */
+  struct env_set *es;
+  const struct plugin_list *plugins;
+
+  /* --gremlin bits */
+  int gremlin;
 };
 
 /* index into tls_session.key */
@@ -319,6 +482,14 @@ struct tls_session
 
   int limit_next;               /* used for traffic shaping on the control channel */
 
+  int verify_maxlevel;
+
+  char *common_name;
+  bool verified;                /* true if peer certificate was verified against CA */
+
+  /* not-yet-authenticated incoming client */
+  struct sockaddr_in untrusted_sockaddr;
+
   struct key_state key[KS_SIZE];
 };
 
@@ -345,6 +516,9 @@ struct tls_session
  */
 struct tls_multi
 {
+  /* used to coordinate access between main thread and TLS thread */
+  //MUTEX_PTR_DEFINE (mutex);
+
   /* const options and config info */
   struct tls_options opt;
 
@@ -367,13 +541,14 @@ struct tls_multi
 
   /*
    * Number of errors.
-   *
-   * Includes:
-   *   (a) errors due to TLS negotiation failure
-   *   (b) errors due to unrecognized or failed-to-authenticate
-   *       incoming packets
    */
-  int n_errors;
+  int n_hard_errors;   /* errors due to TLS negotiation failure */
+  int n_soft_errors;   /* errors due to unrecognized or failed-to-authenticate incoming packets */
+
+  /*
+   * Our locked common name (cannot change during the life of this tls_multi object)
+   */
+  char *locked_cn;
 
   /*
    * Our session objects.
@@ -381,18 +556,31 @@ struct tls_multi
   struct tls_session session[TM_SIZE];
 };
 
+/*
+ * Used in --mode server mode to check tls-auth signature on initial
+ * packets received from new clients.
+ */
+struct tls_auth_standalone
+{
+  struct key_ctx_bi tls_auth_key;
+  struct crypto_options tls_auth_options;
+  struct frame frame;
+};
+
 void init_ssl_lib (void);
 void free_ssl_lib (void);
 
 /* Build master SSL_CTX object that serves for the whole of openvpn instantiation */
-SSL_CTX *init_ssl (bool server,
-		   const char *ca_file,
-		   const char *dh_file,
-		   const char *cert_file,
-		   const char *priv_key_file, const char *cipher_list);
+SSL_CTX *init_ssl (const struct options *options);
 
 struct tls_multi *tls_multi_init (struct tls_options *tls_options);
 
+struct tls_auth_standalone *tls_auth_standalone_init (struct tls_options *tls_options,
+						      struct gc_arena *gc);
+
+void tls_auth_standalone_finalize (struct tls_auth_standalone *tas,
+				   const struct frame *frame);
+
 void tls_multi_init_finalize(struct tls_multi *multi,
 			     const struct frame *frame);
 
@@ -403,17 +591,19 @@ void tls_multi_init_set_options(struct tls_multi* multi,
 bool tls_multi_process (struct tls_multi *multi,
 			struct buffer *to_link,
 			struct sockaddr_in *to_link_addr,
-			struct link_socket *to_link_socket,
-			interval_t *wakeup,
-			time_t current);
+			struct link_socket_info *to_link_socket_info,
+			interval_t *wakeup);
 
 void tls_multi_free (struct tls_multi *multi, bool clear);
 
 bool tls_pre_decrypt (struct tls_multi *multi,
 		      struct sockaddr_in *from,
 		      struct buffer *buf,
-		      struct crypto_options *opt,
-		      time_t current);
+		      struct crypto_options *opt);
+
+bool tls_pre_decrypt_lite (const struct tls_auth_standalone *tas,
+			   const struct sockaddr_in *from,
+			   const struct buffer *buf);
 
 void tls_pre_encrypt (struct tls_multi *multi,
 		      struct buffer *buf, struct crypto_options *opt);
@@ -423,68 +613,46 @@ void tls_post_encrypt (struct tls_multi *multi, struct buffer *buf);
 void show_available_tls_ciphers (void);
 void get_highest_preference_tls_cipher (char *buf, int size);
 
+void pem_password_setup (const char *auth_file);
 int pem_password_callback (char *buf, int size, int rwflag, void *u);
+void auth_user_pass_setup (const char *auth_file);
+void ssl_set_auth_nocache (void);
 
 void tls_set_verify_command (const char *cmd);
 void tls_set_crl_verify (const char *crl);
 void tls_set_verify_x509name (const char *x509name);
-int get_max_tls_verify_id ();
 
 void tls_adjust_frame_parameters(struct frame *frame);
 
+bool tls_send_payload (struct tls_multi *multi,
+		       const uint8_t *data,
+		       int size);
+
+bool tls_rec_payload (struct tls_multi *multi,
+		      struct buffer *buf);
+
+const char *tls_common_name (struct tls_multi* multi, bool null);
+void tls_set_common_name (struct tls_multi *multi, const char *common_name);
+void tls_lock_common_name (struct tls_multi *multi);
+
+bool tls_authenticated (struct tls_multi *multi);
+void tls_deauthenticate (struct tls_multi *multi);
+
 /*
- * TLS thread mode
+ * inline functions
  */
-#ifdef USE_PTHREAD
 
-#define TTCMD_PROCESS 0
-#define TTCMD_EXIT    1
-
-struct tt_cmd
+static inline int
+tls_test_payload_len (const struct tls_multi *multi)
 {
-  int cmd;
-};
-
-struct tt_ret
-{
-  struct buffer to_link;
-  struct sockaddr_in to_link_addr;
-};
-
-struct thread_parms
-{
-# define TLS_THREAD_MAIN   0
-# define TLS_THREAD_WORKER 1
-# define TLS_THREAD_SOCKET(x) ((x)->sd[TLS_THREAD_MAIN])
-
-  /* these macros are called in the context of the openvpn() function */
-# define TLS_THREAD_SOCKET_ISSET(tp, set) (tls_multi && FD_ISSET (TLS_THREAD_SOCKET (&tp), &event_wait.set))
-# define TLS_THREAD_SOCKET_SET(tp, set) { if (tls_multi) FD_SET (TLS_THREAD_SOCKET (&tp), &event_wait.set); }
-# define TLS_THREAD_SOCKET_SETMAXFD(tp) \
-  { if (tls_multi) wait_update_maxfd (&event_wait, TLS_THREAD_SOCKET (&tp)); }
-
-  int sd[2];
-
-  struct tls_multi *multi;
-  struct link_socket *link_socket;
-  int nice;
-  bool mlock;
-};
-
-void tls_thread_create (struct thread_parms *state,
-			struct tls_multi *multi,
-			struct link_socket *link_socket,
-			int nice, bool mlock);
-
-int tls_thread_process (struct thread_parms *state);
-
-void tls_thread_close (struct thread_parms *state);
-
-int tls_thread_rec_buf (struct thread_parms *state,
-			struct tt_ret* ttr,
-			bool do_check_status);
-
-#endif
+  if (multi)
+    {
+      const struct key_state *ks = &multi->session[TM_ACTIVE].key[KS_PRIMARY];
+      if (ks->state >= S_ACTIVE)
+	return BLEN (&ks->plaintext_read_buf);
+    }
+  return 0;
+}
 
 /*
  * protocol_dump() flags
@@ -494,7 +662,9 @@ int tls_thread_rec_buf (struct thread_parms *state,
 #define PD_TLS                     (1<<9)
 #define PD_VERBOSE                 (1<<10)
 
-const char *protocol_dump (struct buffer *buffer, unsigned int flags);
+const char *protocol_dump (struct buffer *buffer,
+			   unsigned int flags,
+			   struct gc_arena *gc);
 
 /*
  * debugging code
@@ -504,4 +674,9 @@ const char *protocol_dump (struct buffer *buffer, unsigned int flags);
 void show_tls_performance_stats(void);
 #endif
 
+//#define EXTRACT_X509_FIELD_TEST
+void extract_x509_field_test (void);
+
 #endif /* USE_CRYPTO && USE_SSL */
+
+#endif
